
Ext.define('DSS.addon.EventMailer.GridPanel', {

    extend : 'Ext.grid.Panel',

    config : {
        enableColumnHide : false,
        enableColumnMove : false,
        selType : 'rowmodel',
        viewConfig : {
            forceFit : true
        }
    },

    /** The event store */
    store : null,
    editorWindow : null,
    curSelectedEvent : null,
    curSelectedEventId : null,
    btnNew : null,
    btnEdit : null,
    btnDelete : null,
    btnDisable : null,
    btnEnable : null,
    btnTest : null,

    /** DSS JSon class */
    property : 'DSS.json.Property',
    
    editSelectedRecord: function ()  {
        var oParams = {};
        oParams.id = this.curSelectedEvent.get('id');
        oParams.name = this.curSelectedEvent.get('name');
        oParams.triggers = this.curSelectedEvent.get('triggers');
        oParams.message = this.curSelectedEvent.get('message');
        oParams.defaultFooter = this.curSelectedEvent.get('defaultFooter');
        this.editorWindow.show();
        this.editorWindow.setValues(oParams);
    },
    
    constructor : function(config) {

        var me = this;

        this.initConfig(config);
        this.property = Ext.create(this.property, config);

        this.btnNew = new Ext.create('Ext.button.Button', {
            text : _('Neue Nachricht'),
            iconCls : 'icon-add-action',
            handler : function() {
                //me.curSelectedEvent = null;
                //me.curSelectedEventId = null;
                me.editorWindow.show();
                me.editorWindow.setValues(null);
            }
        });
        this.btnEdit = new Ext.create('Ext.button.Button', {
            text : _('Nachricht bearbeiten'),
            iconCls : 'icon-edit-action',
            disabled : true,
            handler : function() {
                me.editSelectedRecord();
            }
        });
        this.btnDelete = new Ext.create('Ext.button.Button', {
            text : _('Nachricht l&ouml;schen'),
            iconCls : 'icon-delete-action',
            disabled : true,
            handler : function() {
                dss.raiseEvent('event-mailer', {
                    actions : 'delete',
                    id : me.curSelectedEvent.get('id')
                });
                me.waitAndReload();
            }
        });
        this.btnDisable = new Ext.create('Ext.button.Button', {
            text : _('Nachricht deaktivieren'),
            iconCls : 'icon-disable-action',
            disabled : true,
            handler : function() {
                dss.raiseEvent('event-mailer', {
                    actions : 'disable',
                    id : me.curSelectedEvent.get('id')
                });
                me.waitAndReload();
            }
        });
        this.btnEnable = new Ext.create('Ext.button.Button', {
            text : _('Nachricht aktivieren'),
            iconCls : 'icon-enable-action',
            hidden : true,
            disabled : true,
            handler : function() {
                dss.raiseEvent('event-mailer', {
                    actions : 'enable',
                    id : me.curSelectedEvent.get('id')
                });
                me.waitAndReload();
            }
        });
        this.btnTest = new Ext.create('Ext.button.Button', {
            text : _('Nachricht testen'),
            iconCls : 'icon-test-action',
            disabled : true,
            handler : function() {
                dss.raiseEvent('event-mailer', {
                    actions : 'test',
                    id : me.curSelectedEvent.get('id')
                });
            }

        });

        Ext.apply(this, {
            tbar : {
                items : [ this.btnNew, this.btnEdit, this.btnDisable, this.btnEnable, this.btnTest, '->', this.btnDelete ]
            }
        });

        this.callParent(arguments);
    },
    waitAndReload : function() {
        var me = this;
        var iSubscriptID = Math.floor(Math.random() * 100000000000);
        dss.ajaxSyncRequest('/json/event/subscribe', {
            subscriptionID : iSubscriptID,
            name : 'event-mailer-saved'
        });
        dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
            subscriptionID : iSubscriptID,
            timeout : 60 * 1000
        }, function() {
            me.loadParameter();
            dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
                subscriptionID : iSubscriptID,
                name : 'event-mailer-saved'
            });
        });
    },

    loadParameter : function(me) {
        if (me == null)
            me = this;
        var sString = dss.ajaxSyncRequest('/json/property/query', {
            'query' : '/scripts/event-mailer/values/*(*)/*(*)/*(*)'
        });
        var result = Ext.JSON.decode(sString);
        
        oResultObject=dss.parsePropertyQueryAnswer(result,[{name:'name',type:'simple'},
                                                           {name:'id',type:'simple'},
                                                           {name:'disabled',type:'condition/enabled'},
                                                           {name:'message', type:'message'},
                                                           {name:'triggers',type:'trigger'}]);    
        if (oResultObject!=null) {
            me.store.removeAll();
            me.store.add(oResultObject);
            if (me.curSelectedEventId != null) {
                var oToSelect = me.store.findRecord('id', me.curSelectedEventId);
                if (oToSelect != null) {
                    me.getSelectionModel().select(oToSelect);
                    me.curSelectedEvent = oToSelect;
                }
            }
        }
    },
    initComponent : function() {
        var me = this;

        me.store = Ext.create('Ext.data.Store', {
            storeId : 'emailStore',
            fields : [ {name:'disabled', sortType:'asBoolean'}, 
                       {name:'name', sortType:'asUCText'}, 
                       {name:'id'}, 
                       {name:'message', sortType:'asEmail'}, 
                       {name:'triggers', sortType:'asTrigger'}, 
                       {name:'defaultFooter', sortType:'asBoolean'} ],
            proxy : {
                type : 'memory',
                reader : {
                    type : 'json',
                    root : 'items'
                }
            }
        });

        var editorConfig = me.config;
        editorConfig.title = 'Nachrichteneigenschaften';
        editorConfig.staticControls = [ {
            type : 'text',
            label : _('Name'),
            id : 'name'
        }, {
            type : 'hidden',
            label : _('id'),
            id : 'id'
        } ];
        editorConfig.tabControls = [ {
            tabLabel : 'Ausl&ouml;ser',
            tabDescription : 'W&auml;hlen Sie die Aktivit&auml;t aus, welche als Ausl&ouml;ser dienen soll. Es k&ouml;nnen gleichzeitig mehrere Stimmungen ausgew&auml;hlt werden.',
            content : [ {
                type : 'TriggerConfigurator',
                id : 'triggers',
                flex : true
            } ]
        }, {
            tabLabel : 'Bedingungen',
            content : []
        }, {
            tabLabel : 'Nachricht',
            tabDescription : 'Geben Sie die Nachricht ein, welche verschickt werden soll.',
            content : [ {
                type : 'MessageControl',
                id : 'message',
                title : "Nachricht",
                flex : true
            }/*, {
                type : 'checkbox',
                id : 'defaultFooter',
                label : "Standardtext mit Beschreibung des ausl&ouml;senden Events anf&uuml;gen"
            } */]
        } ];

        me.editorWindow = Ext.create('DSS.component.Editor', editorConfig);
        me.on({
            selectionchange : function(view, selections, options) {
                var me = this;
                if (selections.length > 0) {
                    me.curSelectedEvent = selections[0];
                    me.curSelectedEventId = selections[0].get('id');
                    me.btnEdit.setDisabled(false);
                    me.btnDelete.setDisabled(false);
                    me.btnDisable.setDisabled(me.curSelectedEvent.get('disabled'));
                    me.btnDisable.setVisible(!me.curSelectedEvent.get('disabled'));
                    me.btnEnable.setDisabled(!me.curSelectedEvent.get('disabled'));
                    me.btnEnable.setVisible(me.curSelectedEvent.get('disabled'));
                    me.btnTest.setDisabled(me.curSelectedEvent.get('disabled'));
                } else {
                    me.curSelectedEvent = null;
                    me.btnEdit.setDisabled(true);
                    me.btnDelete.setDisabled(true);
                    me.btnDisable.setDisabled(true);
                    me.btnEnable.setDisabled(true);
                    me.btnEnable.setVisible(false);
                    me.btnTest.setDisabled(true);
                }
            },          
            itemdblclick: function() {
                var me = this;
                if (me.curSelectedEvent != null) {
                    me.editSelectedRecord();
                };
            }
        });
        me.editorWindow.on({
            'save' : function(oValue) {
                dss.raiseEvent('event-mailer', {
                    actions : 'save',
                    value : Ext.JSON.encode(oValue)
                })
                me.waitAndReload();
            }
        })
        me.callParent(arguments);
        me.initPage();
        me.loadParameter();
    }, // initComponent

    initPage : function() {
    },

    viewConfig : {
        stripeRows : true
    },

    columns : [
            {
                id : 'col-status',
                header : _('Aktiv'),
                dataIndex : 'disabled',
                xtype: 'disabledcolumn',                 
                resizable : false,
                width : 50,
                listeners: {                  
                    checkchange : function (column,oValue,dataIndex, checked) {
                        if (oValue.get('disabled')==false)
                        {
                            dss.raiseEvent('event-mailer', {
                                actions : 'enable',
                                id : oValue.get('id')                        
                            });  
                            this.up('grid').waitAndReload();
                        } else {
                            dss.raiseEvent('event-mailer', {
                                actions : 'disable',
                                id : oValue.get('id')
                            });
                            this.up('grid').waitAndReload();
                        }
                    }
                }
            },
            {
                id : 'col-name',
                header : _('Name'),
                dataIndex : 'name',
                xtype: 'textcolumn',
                flex : 1
            },
            {
                id : 'col-actions',
                header : _('Ausl&ouml;ser'),
                dataIndex : 'triggers', // alibi index
                maintainFlex : true,
                flex : 1,
                xtype: 'triggercolumn'
            },
            {
                id : 'col-message',
                header : _('Nachricht'),
                dataIndex : 'message',
                xtype:'emailcolumn',
                flex : 1
            },

    ]
});

function pad(c) {
    var i = parseInt(c, 10);
    return(i < 10 ? '0' + i : c.toString());
}
